######################
# Project: The potential distribution of Ixodes cookei 
######################


# packages
## install


if(!require(dismo)){
  install.packages("dismo")
  library(dismo)
}

if(!require(raster)){
  install.packages("raster")
  library(raster)
}

if(!require(sp)){
  install.packages("sp")
  library(sp)
}

if(!require(maptools)){
  install.packages("maptools")
  library(maptools)
}

if(!require(maps)){
  install.packages("maps")
  library(maps)
}

if(!require(rgeos)){
  install.packages("rgeos")
  library(rgeos)
}

if(!require(rgdal)){
  install.packages("rgdal")
  library(rgdal)
}

if(!require(devtools)){
  install.packages("devtools")
  library(devtools)
}
install.packages("backports")
install.packages("dismo")
install.packages("raster")
install.packages("sp")
install.packages("maptools")
if(!require(maps)){
  install.packages("maps")
  library(maps)
}

## load
library(dismo)
library(maptools)

# directory
setwd("D:/I.cookei/")

## load data
IC <- read.csv("IC_data_cleaned.csv")


# Package needed
if(!require(spThin)){
  install.packages("spThin")
  library(spThin)
}


# directory 
setwd("C:/Ixodes cookei")

IC1 <- read.csv("IC_data_cleaned.csv")


# Thining
help(thin) # function's help

thinnedIC1 <- thin( loc.data = IC1, lat.col = "lat", long.col = "lon", 
                   spec.col = "species", thin.par = 50, reps = 50, 
                   locs.thinned.list.return = TRUE, write.files = TRUE, 
                   max.files = 1, out.dir = "C:/I.cookei/thinning_data50km", 
                   out.base = "IC_thined", write.log.file = TRUE,
                   log.file = "IC_thined_full_log_file.txt" )


######################
# Process: Design M areas and download environmental variable
######################


## Design the M

### create buffers of () km

#### convert points to spatial points

## first species ##

IC <- read.csv("thinning_data50km/IC2_thined_thin1.csv")
WGS84 <- sp::CRS("+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0")
occ_sp <- sp::SpatialPointsDataFrame(coords = IC[, 2:3], data = IC,
                                     proj4string = WGS84)

plot(occ_sp)
map("world", add = T)


#### project from log and lat to projections which keeps area correct.

centroid <- rgeos::gCentroid(occ_sp, byid = FALSE)

AEQD <- sp::CRS(paste("+proj=aeqd +lat_0=", centroid@coords[2], " +lon_0=", centroid@coords[1],
                      " +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs", sep = ""))

occ_pr <- sp::spTransform(occ_sp, AEQD)



#### creat a buffer

buff_area <- rgeos::gBuffer(occ_pr, width = 500000) ## 500000 meters buffer distance

buff_area1 <- raster::disaggregate(buff_area)


#### save shapefile

## Create file
dir.create("M_area")

occ_prIC <- SpatialPolygonsDataFrame(occ_prIC, data = data.frame(c(500, 500)), match.ID = F)


rgdal::writeOGR(occ_prIC, ".", "M_area/M_IC", driver = "ESRI Shapefile")


## Download variables
WC_data <- getData('worldclim', var='bio', res=10)
plot(WC_data[[1]])


##### Change Woldclim formate to ASCII ########

# Mask all worldclim variables #

maskWC <- WC_data[[c(-8, -9, -18, -19)]]

## save data
dir.create("WD_asc_r")

nameWC <- c(paste("WD_asc_r", paste0(gsub("bio", "bio_", names(maskWC[[1:9]])), ".asc"), sep = "/"), 
            paste("WD_asc_r", paste0(gsub("bio", "bio_", names(maskWC[[10:15]])), ".asc"), sep = "/"))


for (i in 1:length(nameWC)) {
  writeRaster(maskWC[[i]], nameWC[i], format = "ascii", NAflag = -9999)
  
}

#############

## Mask variables ## first we use crop then we mask the area ##
maskIC <- mask(crop(WC_data, occ_prIC), occ_prIC)

maskIC <- maskIC[[c(-8, -9, -18, -19)]]

## save data
dir.create("IC_ENM")
dir.create("IC_ENM/variablesM")

nameIC <- c(paste("IC_ENM/variablesM", paste0(gsub("bio", "bio_", names(maskIC[[1:9]])), ".asc"), sep = "/"), 
            paste("IC_ENM/variablesM", paste0(gsub("bio", "bio_", names(maskIC[[10:15]])), ".asc"), sep = "/"))

for (i in 1:length(nameIC)) {
  writeRaster(maskIC[[i]], nameIC[i], format = "ascii")

}

######################
# Process: Select variables (PCA) for I. cookie
######################


suppressWarnings({
  if(!require(raster)){
    install.packages("raster")
    library(raster)
  }
}) 

# loading needed package
# assuming that you installed kuenm, load it, if not installed see 
# https://github.com/marlonecobos/kuenm for instructions
library(kuenm)

install.packages("remotes")
remotes::install_github("manubio13/ku.enm")

setwd("D:/New_Phd_work/I.cookie/")

## Select variables (PCA)

var_folder <- "IC_ENM/variablesM" # name of folder with variables to be combined in distinct sets
proj_folder <- "IC_ENM/var_f" # name of the folder containing one or more folders with variables for other scenarios
out_folder <- "IC_ENM/PCA_results" # name of folder that will contain the sets 
in_format <- "ascii" # other options available are "GTiff" and "EHdr" = bil 
out_format <- "ascii" # other options available are "GTiff" and "EHdr" = bil
n_pcs <- 5 # number of pcs you want as rasters, if not defined all pcs are returned as rasters
scalev <- TRUE # scale variables
writer <- TRUE # save results
project <- TRUE
out_format <- "ascii"


 kuenm_rpca(variables = var_folder, in.format = in_format,var.scale = scalev, 
            write.result = writer, out.format = out_format, project = project, 
           proj.vars = proj_folder, n.pcs = n_pcs, out.dir = out_folder)


 # Process: Preparing data for Ecological niche modeling for i. cookie.
 
 # Ecological niche modeling
 
 ## Prepare the data
 
 ### M variables
 dir.create("IC_ENM/M_variables")
 
 #### manually done
 
 ## second species ## 
 
 dir.create("C:/I.cookei/")
 
 #### manually done
 
 
 ## occurrences out of rasters
 
 MlayerIC <- raster("IC_ENM/M_variables/Set_1/pc_1.asc")

 plot(MlayerIC)
 points(IC[, 2:3])
 
 
 ICcheck <- data.frame(IC, value = extract(MlayerIC, IC[, 2:3]))
 ## IAcheck <- data.frame(IA, value = extract(MlayerIA, IA[, 2:3]))
 
 ICout <- ICcheck[is.na(ICcheck$value), ]

 plot(MlayerIC)
 points(ICout[, 2:3])
 
 ICcheck <- ICcheck[!is.na(ICcheck$value), ]
 
 
 
 ### occrence data
 
 ICall <- ICcheck[, -4]
 ICall$code <- paste0(ICall$species, ICall$log, ICall$lat) 
 ICtrain <- ICall[ICall$code %in% sample(ICall$code, length(ICall$code)/2), ]
 ICtest <- ICall[!ICall$code %in% ICtrain$code, ]
 
 
 write.csv(ICall, "IC_ENM/IC_joint.csv", row.names = F)
 write.csv(ICtrain, "IC_ENM/IC_train.csv", row.names = F)
 write.csv(ICtest, "IC_ENM/IC_test.csv", row.names = F)
 
 
 
 # Ecological niche modeling
 
 ## Model calibration
 
 # Installing and loading packages
 
 setwd("C:/PhD_projects/Ixodes cookei")
 
 if(!require(devtools)){
   install.packages("devtools")
 }
 
 if(!require(kuenm)){
   devtools::install_github("marlonecobos/kuenm")
 }
 
 library(kuenm)
 
 ### Candidate models
 
 spfolders <- "IC_ENM/"
 
 naminfol <- "IC"
 
 occ_joint <- paste0(spfolders, naminfol, "_joint.csv")
 occ_tra <-  paste0(spfolders, naminfol, "_train.csv")
 M_var_dir <-  paste0(spfolders, "M_variables") 
 batch_cal <- paste0(spfolders, "Candidate_models") 
 out_dir <- paste0(spfolders, "Candidate_Models")
 reg_mult <- c(seq(0.1, 1, 0.1), seq(2, 6, 1), 8, 10)
 f_clas <- "no.t"
 # background <- 10000
 maxent_path <- "C:/Maxent"
 wait <- FALSE
 run <- TRUE
 
 occ_test <- paste0(spfolders, naminfol, "_test.csv")
 out_eval <- paste0(spfolders, "Calibration_results")
 threshold <- 5
 rand_percent <- 50
 iterations <- 500
 kept <- TRUE
 selection <- "OR_AICc"
 paral_proc <- FALSE
 
 for (i in 1:length(occ_joint)) {
   kuenm_cal(occ.joint = occ_joint[i], occ.tra = occ_tra[i], M.var.dir = M_var_dir[i], batch = batch_cal[i],
             out.dir = out_dir[i], reg.mult = reg_mult, f.clas = f_clas,
             maxent.path = maxent_path, wait = wait, run = run)
   
   ### Candidate model evaluation and selection
   
   kuenm_ceval(path = out_dir[i], occ.joint = occ_joint[i], occ.tra = occ_tra[i], occ.test = occ_test[i], 
               batch = batch_cal[i], out.eval = out_eval[i], threshold = threshold, rand.percent = rand_percent, 
               iterations = iterations, kept = kept, selection = selection, parallel.proc = paral_proc)
   
 }
 
 
 ##### crop future env data to North America
 
 
 # Loading library
 library(raster)
 
 # create G folder
 dir.create("IC_ENM/G_variables")
 
 # put sets inside G folder using loop
 
 for (i in 1:6) {
   dir.create(paste0("IC_ENM/G_variables/Set_", i))
 }
 
 ## reading the future data files
 folders <- dir(path = "IC_ENM/PCA_results/", full.names = T)
 folders1 <- dir(path = "IC_ENM/PCA_results/", full.names = F)
 
 Ra <- raster("IC_ENM/PCA_results/cccma_45/pc_1.asc")
 

 ## crop 
 ex <- extent(-168.011, -51.961, 6.996, 83.433)
 # plot(crop(Ra, ex))
 
 #names of vars in sets
 nam_set <- list(s1 = list.files(path = "IC_ENM/M_variables/Set_1", pattern = "asc$"),
                 s2 = list.files(path = "IC_ENM/M_variables/Set_2", pattern = "asc$"),
                 s3 = list.files(path = "IC_ENM/M_variables/Set_3", pattern = "asc$"),
                 s4 = list.files(path = "IC_ENM/M_variables/Set_4", pattern = "asc$"),
                 s5 = list.files(path = "IC_ENM/M_variables/Set_5", pattern = "asc$"),
                 s6 = list.files(path = "IC_ENM/M_variables/Set_6", pattern = "asc$"))
 
 ## creating loop to crop Future env data to North America. 
 
 for (i in 1:length(folders)) {
   variables <- list.files(path = folders[i], pattern = "asc$", full.names = T)
   st <- stack(variables)
   st <- crop(st, ex)
   
   for (j in 1:length(nam_set)) {
     infolder <- paste0("IC_ENM/G_variables/Set_", j, "/", folders1[i])
     dir.create(infolder)
     varset <- gsub(".asc", "", nam_set[[j]])
     st1 <- st[[varset]]
     varnames <- paste0(names(st1), ".asc")
     
     for (k in 1:nlayers(st1)) {
       writeRaster(st1[[k]], filename = paste0(infolder, "/", varnames[k]), format = "ascii")
     }
   }
   cat(i,"of", length(folders), "\n")
 }
 
 
 
 
 ## Final models and projections
 
 batch_fin <- paste0(spfolders, "Final_models")
 mod_dir <- paste0(spfolders, "Final_Models")
 rep_n <- 10
 rep_type <- "Bootstrap"
 jackknife <- TRUE
 out_format <- "logistic"
 project <- TRUE
 G_var_dir <- paste0(spfolders, "G_variables")
 ext_type <- "all"
 write_mess <- FALSE
 write_clamp <- FALSE
 wait1 <- FALSE
 run1 <- TRUE
 args <- NULL 
 
 for (i in 1:length(occ_joint)) {
   kuenm_mod(occ.joint = occ_joint[i], M.var.dir = M_var_dir[i], out.eval = out_eval[i], batch = batch_fin[i], rep.n = rep_n,
             rep.type = rep_type, jackknife = jackknife, out.dir = mod_dir[i], out.format = out_format, project = project,
             G.var.dir = G_var_dir[i], ext.type = ext_type, write.mess = write_mess, write.clamp = write_clamp, 
             maxent.path = maxent_path, args = args, wait = wait1, run = run1)
   
 }
 
 
 ## Changes in suitable areas and suitability, Uncertainty
 
 format <- "asc"
 stats <- c("med", "range")
 rep <- TRUE
 scenarios <- list(c("present", "cccma_45", "cccma_85", "cesm1_45", "cesm1_85", 
                     "giss_45", "giss_85", "mohc_45", "mohc_85", 
                     "ncar_45", "ncar_85"))
 
 ext_type <- c("E", "EC", "NE") # the type of extrapolation can be selected according to user requirements 
 out_dir1 <- paste0(spfolders, "Final_Model_Stats")
 
 fmod_stats <- paste0(spfolders, "Final_Model_Stats")
 thres <- 5
 curr <- "present"
 emi_scenarios <- c("45", "85")
 c_mods <- c("cccma", "cesm1", "giss", "mohc", "ncar")
 ext_type <- c("E", "EC", "NE")
 out_dir2 <- paste0(spfolders, "Projection_Changes") 
 
 split <- 100
 out_dir3 <- paste0(spfolders, "Variation_from_sources")
 
 for (i in 1:length(occ_joint)) {
   sp_name <- as.character(read.csv(occ_joint[i])[1, 1])
   
   # model statistic calculation
   kuenm_modstats(sp.name = sp_name, fmod.dir = mod_dir[i], format = format, project = project, 
                  statistics = stats, replicated = rep, proj.scenarios = scenarios[[i]], 
                  ext.type = ext_type, out.dir = out_dir1[i])
   
   # identificaton of changes in suitable areas and suitability
   kuenm_projchanges(occ = occ_joint[i], fmod.stats = fmod_stats[i], threshold = thres, current = curr, 
                     emi.scenarios = emi_scenarios, clim.models = c_mods, ext.type = ext_type, 
                     out.dir = out_dir2[i])
   
   # measuring and maping model variability
   kuenm_modvar(sp.name = sp_name, fmod.dir = mod_dir[i], replicated = rep, format = format,  
                project = project, current = curr, emi.scenarios = emi_scenarios, 
                clim.models = c_mods, ext.type = ext_type, split.length = split, out.dir = out_dir3[i])
   
   cat("#########========= species", i, "of 2 finished\n")
 }
 
 ## Strict extrapolation
 help(kuenm_mmop)
 
 
 sets_var <- "Set_6" # a vector of various sets can be used
 out_mop <- paste0(spfolders, "MOP_results")
 percent <- 5
 paral <- TRUE # make this true to perform MOP calculations in parallel, recommended
 # only if a powerfull computer is used (see function's help)
 # Two of the variables used here as arguments were already created for previous functions
 
 
 
 for (i in 1:length(occ_joint)) {
   kuenm_mmop(G.var.dir = G_var_dir[i], M.var.dir = M_var_dir[i], sets.var = sets_var[[i]], out.mop = out_mop[i],
              percent = percent, parallel = paral)
   
   cat("#########========= species", i, "of 2 finished\n")
 }
 
 
 ## agreement of extrapolative areas
 source("kuenm_mopagree.R")
 
 # Arguments
 inout_format <- "GTiff"
 out_magree <- paste0(spfolders, "MOP_agremment")
 
 for (i in 1:length(occ_joint)) {
   kuenm_mopagree(mop.dir = out_mop[i], in.format = inout_format, out.format = inout_format,
                  current = curr, emi.scenarios = emi_scenarios, out.dir = out_magree[i])
   
   cat("#########========= species", i, "of 2 finished\n")
 }
 












